package envoy.server.data;

import java.util.Date;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

/**
 * This class acts as a superclass for all contacts, being {@link User}s and
 * {@link Group}s. <br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>Contact.java</strong><br>
 * Created: <strong>24.03.2020</strong><br>
 *
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-alpha
 */

@Entity
@Table(name = "contacts")
@Inheritance(strategy = InheritanceType.JOINED)
public abstract class Contact {

	@Id
	@GeneratedValue
	protected long		id;
	protected String	name;

	@Temporal(TemporalType.TIMESTAMP)
	private Date creationDate;

	@ManyToMany(cascade = CascadeType.ALL, fetch = FetchType.EAGER)
	protected Set<Contact> contacts;

	/**
	 * @return a {@link envoy.data.Contact} object of this envoy.server.data.Contact
	 *         object.
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public abstract envoy.data.Contact toCommon();

	/**
	 * Transforms this contact into a {@link envoy.data.Contact} where the contacts
	 * set of contacts is empty.
	 *
	 * @return a {@link envoy.data.Contact} object of this contact
	 *         object.
	 * @since Envoy Server Standalone v0.1-beta
	 */
	protected abstract envoy.data.Contact toFlatCommon();

	/**
	 * @return the ID of this contact.
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public long getID() { return id; }

	/**
	 * Sets the ID of this contact.
	 *
	 * @param id to set for this contact
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setID(long id) { this.id = id; }

	/**
	 * @return the name of this contact.
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public String getName() { return name; }

	/**
	 * Sets the name of this contact.
	 *
	 * @param name to set for this contact
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setName(String name) { this.name = name; }

	/**
	 * @return the contacts
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public Set<Contact> getContacts() { return contacts; }

	/**
	 * @param contacts the contacts to set
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setContacts(Set<Contact> contacts) { this.contacts = contacts; }

	/**
	 * @return the creationDate
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public Date getCreationDate() { return creationDate; }

	/**
	 * @param creationDate the creationDate to set
	 * @since Envoy Server Standalone v0.1-beta
	 */
	public void setCreationDate(Date creationDate) { this.creationDate = creationDate; }

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() { return String.format("%s[id=%d,name=%s, contacts=%s]", getClass().getSimpleName(), id, name, contacts); }
}
