package envoy.server.processors;

import java.io.IOException;
import java.util.Date;
import java.util.logging.Logger;

import javax.persistence.EntityExistsException;

import envoy.data.Message;
import envoy.data.Message.MessageStatus;
import envoy.event.MessageStatusChangeEvent;
import envoy.server.data.PersistenceManager;
import envoy.server.net.ConnectionManager;
import envoy.server.net.ObjectWriteProxy;
import envoy.util.EnvoyLog;

/**
 * This {@link ObjectProcessor} handles incoming {@link Message}s.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>MessageProcessor.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-alpha
 */
public class MessageProcessor implements ObjectProcessor<Message> {

	private static final Logger logger = EnvoyLog.getLogger(MessageProcessor.class);

	@Override
	public void process(Message message, long socketID, ObjectWriteProxy writeProxy) {
		message.nextStatus();
		ConnectionManager	connectionManager	= ConnectionManager.getInstance();

		sendToUser(connectionManager, message, writeProxy);
		if (message.getStatus() != MessageStatus.SENT) {
			// Sending a messageStatusChangeEvent to the sender
			try {
				writeProxy.write(socketID, new MessageStatusChangeEvent(message));
			} catch (IOException e) {
				logger.warning("Could not send messageStatusChangeEvent to the sender of this message with ID: " + message.getID());
				e.printStackTrace();
			}
		}
		try {
			PersistenceManager.getInstance().addMessage(new envoy.server.data.Message(message));
		} catch (EntityExistsException e) {
			logger.warning("Received a message with an id that already exists");
		}
	}

	private void sendToUser(ConnectionManager connectionManager, Message message, ObjectWriteProxy writeProxy) {
		if (connectionManager.isOnline(message.getRecipientID())) try {
			// If recipient is online, send the message directly
			writeProxy.write(connectionManager.getSocketId(message.getRecipientID()), message);
			// Update the message status to RECEIVED
			message.setReceivedDate(new Date());
			message.nextStatus();
		} catch (IOException e) {
			logger.warning("Recipient online. Failed to send message" + message.getID());
			e.printStackTrace();
		}
	}

	@Override
	public Class<Message> getInputClass() { return Message.class; }
}
